""" Local Skin: Enable change skins on folder
Copyright (c) 2004-2005 Local Skin development team

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

$Id: LocalSkin.py, sandro lourenco aka Zefyrus
"""
__author__ = 'Sandro Loureco aka Zefyrus'
__docformat__ = 'restructuredtext'

from base64 import encodestring
from urllib import quote, unquote
from DateTime import DateTime
from Products.CMFPlone.PropertiesTool import SimpleItemWithProperties
from AccessControl import ClassSecurityInfo
from ZPublisher import BeforeTraverse
import Globals
from Products.CMFCore.CMFCorePermissions import ManageProperties
from Globals import HTMLFile
from zLOG import LOG, ERROR, INFO
import sys
from OFS.PropertyManager import PropertyManager
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.utils import shasattr
from ZPublisher.HTTPRequest import HTTPRequest
import Missing

class LocalSkin(SimpleItemWithProperties):
    '''
    Reads the atrributes from the folder(s) during tranveral and chage its skin
    '''
    id = 'local_skin_tool'
    meta_type = 'Local Skin Tool'
    
    security = ClassSecurityInfo()


    _properties = ({'id':'selectable_skins', 'type': 'multiple selection', 'mode':'w',
                    'label':'List of Avaible Skins','select_variable': 'listAllAvailablesSkins'},
                   {'id':'enabled', 'type': 'boolean', 'mode':'w',
                    'label':'Local Skin on/off'},
                   )

    selectable_skins = ['']
    enabled = False

    security.declarePrivate('listAllAvaiblesSkins')
    def listAllAvailablesSkins(self):
        st = getToolByName(self, 'portal_skins')
        return st.getSkinSelections()

    def __init__(self):
        pass
    
    def __call__(self, container, req):
        '''The __before_publishing_traverse__ hook.'''
        if not self.enabled: return

##        import pdb
##        pdb.set_trace()
        
        subPath = req.other['TraversalRequestNameStack'][:]
        subPath.reverse()

        localSkin = self._getClosestLocalSkin(list(subPath))
        self.setSkin(localSkin)

    security.declarePublic('getLocalSkin')
    def getLocalSkin(self, obj):
        if not self.isLocallySkinable(obj):
            return Missing.Value
        return getattr(obj.aq_base, 'local_skin', Missing.Value)

    def _getClosestLocalSkin(self, subPath):
        subPath = list(subPath)
        rootPath = list(getToolByName(self, 'portal_url').getPortalObject().getPhysicalPath())
        catalog = getToolByName(self, 'portal_catalog')

        while subPath:
            fullPath = "/" . join(rootPath + list(subPath))
            subPath.pop()
            try:
                md = catalog.getMetadataForUID(fullPath)
            except KeyError:
                continue
            if md['getLocalSkin'] is not Missing.Value:
                LOG('localSkin.localSkin', INFO, 'Swithing to "%s" at "%s"' %
                    (md['getLocalSkin'], fullPath))
                return md['getLocalSkin']
        else:
            st = getToolByName(self, 'portal_skins')
            LOG('localSkin.localSkin', INFO, 'Swithing to "%s" at site root' %
                (st.getDefaultSkin()))            
            return st.getDefaultSkin()

    def _hasMetaData(self, obj , catalog):
        try:
            catalog.getMetadataForUID("/".join(obj.getPhysicalPath()))
            return True
        except KeyError:
            return False

    security.declarePrivate('setSkin')
    def setSkin(self, skin):
        """set the locally selected skin
        """
        self.changeSkin(skin)

    security.declareProtected(ManageProperties, 'setLocalSkin')
    def setLocalSkin(self, obj, skin):
        """ sets the local_skin property
        """
        if not shasattr(obj, 'local_skin'):
            obj.manage_addProperty('local_skin', skin, 'string')
        else:
            obj.manage_changeProperties(local_skin=skin)
        obj.reindexObject()

    security.declareProtected(ManageProperties, 'unsetLocalSkin')
    def unsetLocalSkin(self, obj):
        """ unset the local_skin property if present
        """
#        import pdb
#        pdb.set_trace()
        if shasattr(obj, 'local_skin'):
            obj.manage_delProperties(ids=['local_skin'])
        obj.reindexObject()

    security.declarePublic('getClosestValidObject')
    def getClosestSkinableObject(self, context):
        closestFolder = context
        siteRoot = getToolByName(self, 'portal_url').getPortalObject().aq_base
        while not self.isLocallySkinable(closestFolder):
            closestFolder = closestFolder.aq_parent
        if closestFolder.aq_base is siteRoot:
            return None
        return closestFolder

    security.declarePublic('isLocallySkinable')
    def isLocallySkinable(self, obj):
        return (getattr(obj.aq_base, 'isPrincipiaFolderish', False)
                and isinstance(obj, PropertyManager)
                and shasattr(obj, 'reindexObject'))

    # Installation and removal of traversal hooks.
    def manage_beforeDelete(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            BeforeTraverse.unregisterBeforeTraverse(container, handle)

    def manage_afterAdd(self, item, container):
        if item is self:
            handle = self.meta_type + '/' + self.getId()
            container = container.this()
            nc = BeforeTraverse.NameCaller(self.getId())
            BeforeTraverse.registerBeforeTraverse(container, nc, handle)

Globals.InitializeClass(LocalSkin)
