# LocalSkin | Copyright 4freesystem
# see LICENSE.txt for details

import os, sys

if __name__ == '__main__':
    execfile(os.path.join(os.path.dirname(sys.argv[0]), 'framework.py'))

from Products.CMFPlone.tests import PloneTestCase

for product in (
                'LocalSkin',
                'Archetypes',
                'PortalTransforms',
                ):
    PloneTestCase.ZopeTestCase.installProduct(product, 1)
del product

from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.tests.utils import makeContent
from Products.LocalSkin.Extensions.Install import modify_catalog

#PloneTestCase.setupPloneSite()

class LocalSkinTestCase(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.loginPortalOwner()
        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('LocalSkin')
        self.lst = self.portal.local_skin_tool
        self.mt = self.portal.portal_membership
        self.ct = self.portal.portal_catalog
        self.st = self.portal.portal_skins
        self.home = self.mt.getHomeFolder()
        self.folder = makeContent(self.portal, 'Folder', id='folder')
        self.doc = makeContent(self.folder, 'Document', id='doc')
        self.setupSkin()

    def setupSkin(self):
        self.portal._v_skindata = None
        self.portal.setupCurrentSkin()
        
class TestLocalSkin(LocalSkinTestCase):

    def test_isLocalSkinMetaData(self):
        self.failUnless('getLocalSkin' in self.ct.schema(),
                        'getLocalSkin not in %s' % self.ct.schema())

    def test_setAndRemoveLocalSkin(self):
        import Missing
        self.assertEquals(self.ct(id='folder')[0].getLocalSkin, Missing.Value)
        self.failIf(self.folder.hasProperty('local_skin'))
        #adding the local skin
        self.lst.setLocalSkin(self.folder, 'Plone Default')
        self.assertEquals(self.folder.local_skin, 'Plone Default')
        self.assertEquals(self.ct(id='folder')[0].getLocalSkin, 'Plone Default')
        #lets remove it
        self.lst.unsetLocalSkin(self.folder)
        self.failIf(self.folder.hasProperty('local_skin'))
        self.assertEquals(self.ct(id='folder')[0].getLocalSkin, Missing.Value)

    def test_isLocallySkinable(self):
        self.failIf(self.lst.isLocallySkinable(self.doc))
        self.failUnless(self.lst.isLocallySkinable(self.folder))

    def test_getClosestFolder(self):
        pdoc = makeContent(self.portal, 'Document', id='pdoc')
        self.assertEquals(self.lst.getClosestSkinableObject(self.portal), None)
        self.assertEquals(self.lst.getClosestSkinableObject(pdoc), None)
        self.assertEquals(self.lst.getClosestSkinableObject(self.doc), self.folder)
        self.assertEquals(self.lst.getClosestSkinableObject(self.folder), self.folder)

    def getSubPath(self, obj):
        rootPath = self.portal.getPhysicalPath()
        objPath = obj.getPhysicalPath()
        return objPath[len(rootPath):]

    def test_getClosestLocalSkin(self):
        #portal
        subPath = self.getSubPath(self.portal)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), self.st.getDefaultSkin())
        #non skinned folder
        subPath = self.getSubPath(self.folder)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), self.st.getDefaultSkin())
        #skinned folder
        self.lst.setLocalSkin(self.folder, 'Zefyrus')
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus')
        #existing doc on skinned folder
        subPath = self.getSubPath(self.doc)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus')
        subdoc = makeContent(self.folder, 'Document', id='subdoc')
        subPath = self.getSubPath(subdoc)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus')
        subfolder =  makeContent(self.folder, 'Folder', id='subfolder')
        subPath = self.getSubPath(subfolder)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus')
        self.lst.setLocalSkin(subfolder, 'Zefyrus 2')
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus 2')
        #unsetting the skin
        self.lst.unsetLocalSkin(self.folder)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), 'Zefyrus 2')
        self.lst.unsetLocalSkin(subfolder)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), self.st.getDefaultSkin())
        subPath = self.getSubPath(self.doc)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), self.st.getDefaultSkin())
        subPath = self.getSubPath(subdoc)
        self.assertEquals(self.lst._getClosestLocalSkin(subPath), self.st.getDefaultSkin())     

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    for testclass in (
        TestLocalSkin,
        ):
        suite.addTest(makeSuite(testclass))

    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
